package com.lj.pointgrowth.config;

import org.apache.kafka.clients.CommonClientConfigs;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.common.config.SaslConfigs;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.kafka.annotation.EnableKafka;
import org.springframework.kafka.config.ConcurrentKafkaListenerContainerFactory;
import org.springframework.kafka.config.KafkaListenerContainerFactory;
import org.springframework.kafka.core.ConsumerFactory;
import org.springframework.kafka.core.DefaultKafkaConsumerFactory;
import org.springframework.kafka.listener.ConcurrentMessageListenerContainer;
import org.springframework.kafka.listener.ContainerProperties;

import java.util.HashMap;
import java.util.Map;

/**
 * @author 黄树文
 * @date 2020/1/15 16:26
 */
@Configuration
@EnableKafka
public class KafkaConsumerConfig {
    //采用注解读取kafka.properties字段
    @Value("${spring.kafka.listener.bootstrap-servers}")
    private String kafka_bootstrap_servers;

    @Value("${spring.kafka.consumer.enable-auto-commit}")
    private String kafka_consumer_enable_auto_commit;

    @Value("${spring.kafka.consumer.concurrency}")
    private Integer kafka_consumer_concurrency;

    @Value("${spring.kafka.consumer.auto-commit-interval}")
    private String kafka_consumer_auto_commit_interval_ms;

    @Value("${spring.kafka.consumer.group-id}")
    private String kafka_consumer_group_id;

    @Value("${spring.kafka.consumer.session-timeout-ms}")
    private String kafka_consumer_session_timeout_ms;

    @Value("${spring.kafka.consumer.properties.security.protocol}")
    private String security_protocol;

    @Value("${spring.kafka.consumer.properties.sasl.mechanism}")
    private String sasl_mechanism;

    @Value("${spring.kafka.consumer.properties.sasl.jaas.config}")
    private String sasl_jaas_config;


    @Bean
    public KafkaListenerContainerFactory<ConcurrentMessageListenerContainer<String, String>> kafkaListenerContainerFactory() {
        ConcurrentKafkaListenerContainerFactory<String, String> factory = new ConcurrentKafkaListenerContainerFactory<String, String>();
        factory.setConsumerFactory(consumerFactory());
        factory.setConcurrency(kafka_consumer_concurrency);
        factory.getContainerProperties().setPollTimeout(4000);
        return factory;
    }


    @Bean
    public KafkaListenerContainerFactory<ConcurrentMessageListenerContainer<String, String>> batchFactory() {
        ConcurrentKafkaListenerContainerFactory<String, String> factory = new ConcurrentKafkaListenerContainerFactory<String, String>();
        factory.setConsumerFactory(consumerFactory());
        factory.setConcurrency(kafka_consumer_concurrency);
        // 设置批量消费
        factory.setBatchListener(true);
        factory.getContainerProperties().setAckMode(ContainerProperties.AckMode.MANUAL_IMMEDIATE);
        factory.getContainerProperties().setPollTimeout(4000);
        return factory;
    }


    @Bean
    public ConsumerFactory<String, String> consumerFactory() {
        Map<String, Object> properties = new HashMap<String, Object>();
        properties.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, kafka_bootstrap_servers);
        properties.put(ConsumerConfig.ENABLE_AUTO_COMMIT_CONFIG, kafka_consumer_enable_auto_commit);
        properties.put(ConsumerConfig.AUTO_COMMIT_INTERVAL_MS_CONFIG, kafka_consumer_auto_commit_interval_ms);
        properties.put(ConsumerConfig.SESSION_TIMEOUT_MS_CONFIG, kafka_consumer_session_timeout_ms);
        properties.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
        properties.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
        properties.put(ConsumerConfig.GROUP_ID_CONFIG, kafka_consumer_group_id);

        //设置安全验证  此文配置都默认添加账号验证配置，如果kafka服务器没有开启sasl没有开启测无法连接服务器，已配置可以参考，没有配置一定要取消
        properties.put(CommonClientConfigs.SECURITY_PROTOCOL_CONFIG, security_protocol);
        properties.put(SaslConfigs.SASL_MECHANISM, sasl_mechanism);
        properties.put(SaslConfigs.SASL_JAAS_CONFIG, sasl_jaas_config);
        return new DefaultKafkaConsumerFactory<>(properties);
    }

    @Bean
    public KafkaListeners kafkaListeners() {
        return new KafkaListeners();
    }


}
