package com.lj.pointgrowth.config;


import org.apache.kafka.clients.CommonClientConfigs;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.config.SaslConfigs;
import org.apache.kafka.common.serialization.StringSerializer;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.kafka.annotation.EnableKafka;
import org.springframework.kafka.core.DefaultKafkaProducerFactory;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.core.ProducerFactory;
import org.springframework.scheduling.annotation.EnableAsync;

import java.util.HashMap;
import java.util.Map;

/**
 * @author 黄树文
 * @date 2020/1/15 16:25
 */
@Configuration
@EnableAsync
public class KafkaProducerConfig {
    //采用注解读取kafka.properties字段
    @Value("${spring.kafka.listener.bootstrap-servers}")
    private String kafka_bootstrap_servers;

    @Value("${spring.kafka.producer.acks}")
    private String kafka_producer_acks;

    @Value("${spring.kafka.producer.retries}")
    private String kafka_producer_retries;

    @Value("${spring.kafka.producer.linger-ms}")
    private String kafka_producer_linger_ms;

    @Value("${spring.kafka.producer.buffer-memory}")
    private String kafka_producer_buffer_memory;

    @Value("${spring.kafka.producer.batch-size}")
    private String kafka_producer_batch_size;

    @Value("${spring.kafka.producer.defaultTopic}")
    private String kafka_producer_defaultTopic;

    @Value("${spring.kafka.producer.properties.security.protocol}")
    private String security_protocol;

    @Value("${spring.kafka.producer.properties.sasl.mechanism}")
    private String sasl_mechanism;

    @Value("${spring.kafka.producer.properties.sasl.jaas.config}")
    private String sasl_jaas_config;

    @Bean
    public KafkaTemplate<String, String> kafkaTemplate() {
        KafkaTemplate<String, String> kafkaTemplate = new KafkaTemplate<String, String>(producerFactory());
        kafkaTemplate.setDefaultTopic(kafka_producer_defaultTopic);
        return kafkaTemplate;
    }

    @Bean
    public ProducerFactory<String, String> producerFactory() {
        Map<String, Object> properties = new HashMap<String, Object>();
        properties.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, kafka_bootstrap_servers);
        properties.put(ProducerConfig.RETRIES_CONFIG, kafka_producer_retries);
        properties.put(ProducerConfig.BATCH_SIZE_CONFIG, kafka_producer_batch_size);
        properties.put(ProducerConfig.LINGER_MS_CONFIG, kafka_producer_linger_ms);
        properties.put(ProducerConfig.BUFFER_MEMORY_CONFIG, kafka_producer_buffer_memory);
        properties.put(ProducerConfig.ACKS_CONFIG, kafka_producer_acks);
        properties.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
        properties.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, StringSerializer.class);

        //设置安全验证  此文配置都默认添加账号验证配置，如果kafka服务器没有开启sasl没有开启测无法连接服务器，已配置可以参考，没有配置一定要取消
        properties.put(CommonClientConfigs.SECURITY_PROTOCOL_CONFIG, security_protocol);
        properties.put(SaslConfigs.SASL_MECHANISM, sasl_mechanism);
        properties.put(SaslConfigs.SASL_JAAS_CONFIG, sasl_jaas_config);
        return new DefaultKafkaProducerFactory<>(properties);
    }

    @Bean
    public KafkaSender kafkaSender(){
        return new KafkaSender();
    }

}
