package com.lj.pointgrowth.quaetz;


import com.lj.pointgrowth.pojo.PointGrowthRule;
import com.lj.pointgrowth.schedule.ScheduleJobInfo;
import com.lj.pointgrowth.service.PointGrowthRuleService;
import org.quartz.JobDataMap;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;


@Configuration
@EnableScheduling
@Component
public class ScheduleRefreshDatabase {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private PointGrowthRuleService pointGrowthRuleService;

    @Autowired
    private QuartzManager quartzManager;

    @Resource(name = "schedulerFactoryBean")
    private Scheduler scheduler;

    //    // 每隔5s查库，并根据查询结果决定是否重新设置定时任务
    @Scheduled(fixedRate = 3600000)
    public void scheduleUpdateCronTrigger() throws SchedulerException {
        // 读取数据库定时任务信息
        System.out.println("每隔10s查库，并根据查询结果决定是否重新设置定时任务" + LocalDateTime.now().toString());

        List<PointGrowthRule> pointGrowthRules = pointGrowthRuleService.findAll();

        for (PointGrowthRule pointGrowthRule : pointGrowthRules) {
//            System.out.println(pointGrowthRule);
            ScheduleJobInfo info = new ScheduleJobInfo();
            info.setClassName("com.lj.pointgrowth.schedule.ScheduledJob");
            info.setJobName("job" + pointGrowthRule.getId());
            info.setGroupName("group" + pointGrowthRule.getId());
            info.setStatus(pointGrowthRule.getRunning());
//            System.out.println(pointGrowthRule.getTimeTaskCycle());
            info.setCron(pointGrowthRule.getTimeTaskCycle());

            JobDataMap jobDataMap = new JobDataMap();
            jobDataMap.put("pointGrowthRule", pointGrowthRule);

//            System.out.println(!quartzManager.existsJob(info));
            if (!quartzManager.existsJob(info)) {
                logger.info(pointGrowthRule.getId() + "   不存在");
                if (info.getStatus()) {
                    quartzManager.addJob(info, jobDataMap);
                }
            } else {
                logger.info(pointGrowthRule.getId() + "   存在");
                quartzManager.modifyJob(info, jobDataMap);
            }
        }
    }
}
