package com.lj.pointgrowth.service.impl;

import com.alibaba.fastjson.JSON;
import com.lj.pointgrowth.algorithm.SnowflakeIdWorker;
import com.lj.pointgrowth.config.KafkaSender;
import com.lj.pointgrowth.pojo.*;
import com.lj.pointgrowth.service.*;
import com.lj.pointgrowth.vo.OrdersVo;
import org.quartz.JobDataMap;
import org.quartz.JobExecutionContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.SQLException;
import java.util.Date;
import java.util.List;
import java.util.Random;

/**
 * @Author: 黄树文
 * @Date: 2019/12/12 9:44
 */
@Service
public class PointGrowthCalculateServiceImpl implements PointGrowthCalculateService {

    @Autowired
    private OrdersService ordersService;


    @Autowired
    private UserCalculationLogService userCalculationLogService;


    @Autowired
    private WeixinUserInfoService weixinUserInfoService;


    @Autowired
    private KafkaSender kafkaSender;

    /**
     * @param jobExecutionContext
     * @throws InterruptedException
     * @throws SQLException
     * @throws NoSuchFieldException
     * @throws InstantiationException
     * @throws IllegalAccessException
     */
    @Override
    public void pointGrowthCalculate(JobExecutionContext jobExecutionContext) throws SQLException, NoSuchFieldException, InstantiationException, IllegalAccessException, InterruptedException {

        JobDataMap jobDataMap = jobExecutionContext.getJobDetail().getJobDataMap();

        // 获取积分成长值规则
        PointGrowthRule pointGrowthRule = (PointGrowthRule) jobDataMap.get("pointGrowthRule");

        String terminalCode = pointGrowthRule.getTerminalCode();
        String growthRule = pointGrowthRule.getGrowthRule();
        Double growthRuleRate = Double.parseDouble(growthRule.split("\\*")[1]);
        String pointRule = pointGrowthRule.getPointRule();
        double pointRuleRate = Double.parseDouble(pointRule.split("\\*")[1]);

        /**
         * 单线程每次只查询前1000条数据
         */
        // 获取每个平台的记录数
        Integer numOfOrdersByTerminalVo = ordersService.getNumOfOrdersByTerminalVo(Integer.parseInt(terminalCode));

        Integer size = 1000;
        SnowflakeIdWorker snowflakeIdWorker = new SnowflakeIdWorker(Thread.currentThread().getId() % 31, Thread.currentThread().getId() % 30);
        for (int i = 0; i <= numOfOrdersByTerminalVo / size; i++) {
            List<OrdersVo> ordersVos = ordersService.getOrdersByTerminalVo(Integer.parseInt(terminalCode));

//            ordersVos.forEach(ordersVo -> {
//                hadleOrdersVo(ordersVo, growthRuleRate, pointRuleRate, pointGrowthRule, snowflakeIdWorker.nextId().toString());
//            });
            for (OrdersVo ordersVo : ordersVos) {
                hadleOrdersVo(ordersVo, growthRuleRate, pointRuleRate, pointGrowthRule, snowflakeIdWorker.nextId().toString());
            }
            Thread.sleep(new Random(1).nextInt(5000));
        }
    }

    /**
     * 根据订单信息，对用户的积分额和成长值进行计算，把订单日志信息写入es，并记录计算过的订单，写入sqlserver mem_user_calculation_log表中
     *
     * @param ordersVo        订单信息
     * @param growthRuleRate  成长值与订单金额比率
     * @param pointRuleRate   积分与订单金额比率
     * @param pointGrowthRule
     * @param id              雪花算法计算的订单的id
     */
    public void hadleOrdersVo(OrdersVo ordersVo, Double growthRuleRate, double pointRuleRate, PointGrowthRule pointGrowthRule, String id) {
        // 获取微信用户信息
        WeixinUserInfo weixinUserInfo = weixinUserInfoService.findBySecretPhone(ordersVo.getSecretPhone());

        // 如果微信用户信息表（mem_weixin_user_info）没有查到数据，则新建
        if (weixinUserInfo == null) {
            while (true) {
                weixinUserInfo = new WeixinUserInfo().setId(id)
                        .setSecretPhone(ordersVo.getSecretPhone())
                        .setPoint((int) Math.round(ordersVo.getPayMent() * pointRuleRate))
                        .setPointUpdateDate(new Date())
                        .setGrowth((int) Math.round(ordersVo.getPayMent() * growthRuleRate))
                        .setConsignee(ordersVo.getConsignee())
                        .setGrowthUpdateDate(new Date())
                        .setVersion(1);
                // 保存微信用户信息至sqlserver数据库，如果保存失败，则肯能改用户数据被其他线程创建过，则更新该用户数据
                if (!weixinUserInfoService.save(weixinUserInfo)) {
                    weixinUserInfo = saveWeixinUserInfo(ordersVo, growthRuleRate, pointRuleRate);
                    break;
                } else {
                    break;
                }
            }
        } else {
            weixinUserInfo = saveWeixinUserInfo(ordersVo, growthRuleRate, pointRuleRate);
        }

        // 构建积分log对象
        UserPointLog userPointLog = constructUserPointLog(ordersVo, pointRuleRate, pointGrowthRule, id, weixinUserInfo);
        UserGrowthLog userGrowthLog = constructUserGrowthLog(ordersVo, growthRuleRate, pointGrowthRule, id, weixinUserInfo);

        // System.out.println(userGrowthLog);
        // System.out.println(userPointLog);

        UserCalculationLog userCalculationLog = new UserCalculationLog();
        userCalculationLog.setId(userGrowthLog.getId()).setOrdersCode(userGrowthLog.getDataCode())
                .setCrtDate(userGrowthLog.getCrtDate()).setLastUpdate(userGrowthLog.getLastUpdate());

        // 日志写入sqlserver
        userCalculationLogService.save(userCalculationLog);

        // // 成长值log 积分log 结果写入kafka
        saveLogToKafka(userPointLog, userGrowthLog);

    }


    /**
     * 构建userGrowthLog对象
     *
     * @param ordersVo
     * @param growthRuleRate
     * @param pointGrowthRule
     * @param id
     * @param weixinUserInfo
     * @return
     */
    private UserGrowthLog constructUserGrowthLog(OrdersVo ordersVo, Double growthRuleRate, PointGrowthRule pointGrowthRule, String id, WeixinUserInfo weixinUserInfo) {
        UserGrowthLog userGrowthLog = new UserGrowthLog();
        // 构建成长值log对象
        userGrowthLog.setId(id).setCrtDate(ordersVo.getCrtDate()).setLastUpdate(ordersVo.getLastUpdate())
                .setCrtUser("system").setAction("购药").setDataCode(ordersVo.getId()).setSecretPhone(ordersVo.getSecretPhone()).setSecretPhone1(weixinUserInfo.getSecretPhone1())
                .setTerminalAction("购药").setTerminalCode(ordersVo.getTerminalCode().toString()).setTerminalName(ordersVo.getTerminalName()).setIncrease(true)
                .setValue((int) Math.round(ordersVo.getPayMent() * growthRuleRate)).setRemark(pointGrowthRule.getGrowthRule());

        userGrowthLog.setOrigin(weixinUserInfo.getGrowth() - (int) Math.round(ordersVo.getPayMent() * growthRuleRate))
                .setBalance(weixinUserInfo.getGrowth());
        return userGrowthLog;
    }

    /**
     * 构建userPointLog对象
     *
     * @param ordersVo
     * @param pointRuleRate
     * @param pointGrowthRule
     * @param id
     * @param weixinUserInfo
     * @return
     */
    private UserPointLog constructUserPointLog(OrdersVo ordersVo, double pointRuleRate, PointGrowthRule pointGrowthRule, String id, WeixinUserInfo weixinUserInfo) {
        UserPointLog userPointLog = new UserPointLog();
        userPointLog.setId(id).setCrtDate(ordersVo.getCrtDate()).setCrtUser("system").setLastUpdate(ordersVo.getLastUpdate())
                .setAction("购药").setDataCode(ordersVo.getId()).setSecretPhone(ordersVo.getSecretPhone())
                .setSecretPhone1(weixinUserInfo.getSecretPhone1()).setPayment(ordersVo.getPayMent() + 0.0f).setDiscountedPrice(ordersVo.getDiscountPrice() + 0.0f)
                .setIncrease(true).setOrderAmount(ordersVo.getOrderAmount() + 0.0f)
                .setTerminalAction("购药").setTerminalCode(ordersVo.getTerminalCode().toString())
                .setTerminalName(ordersVo.getTerminalName()).setTransportCost(ordersVo.getTransportCosts() + 0.0f)
                .setValue((int) Math.round(ordersVo.getPayMent() * pointRuleRate)).setRemark(pointGrowthRule.getPointRule());
        userPointLog.setOrigin(weixinUserInfo.getPoint() - (int) Math.round(ordersVo.getPayMent() * pointRuleRate))
                .setBalance(weixinUserInfo.getPoint());
        return userPointLog;
    }

    /**
     * 保存weixinUserInfo
     *
     * @param ordersVo
     * @param growthRuleRate
     * @param pointRuleRate
     * @return
     */
    private WeixinUserInfo saveWeixinUserInfo(OrdersVo ordersVo, Double growthRuleRate, double pointRuleRate) {
        WeixinUserInfo weixinUserInfo;
        while (true) {
            weixinUserInfo = weixinUserInfoService.findBySecretPhone(ordersVo.getSecretPhone());
            weixinUserInfo.setSecretPhone(null);
            weixinUserInfo.setPoint(weixinUserInfo.getPoint() + (int) Math.round(ordersVo.getPayMent() * pointRuleRate))
                    .setPointUpdateDate(new Date())
                    .setGrowth(weixinUserInfo.getGrowth() + (int) Math.round(ordersVo.getPayMent() * growthRuleRate))
                    .setGrowthUpdateDate(new Date())
                    .setConsignee(ordersVo.getConsignee());
            try {
                if (weixinUserInfoService.updateById(weixinUserInfo)) {
                    break;
                }
            } catch (Exception e) {
                e.printStackTrace();
                System.out.println(weixinUserInfo);
                System.out.println("-------------------------重试---------------------------------");
            }
        }
        return weixinUserInfo;
    }


    /**
     * 将积分日志和成长值日志表写入kafka
     *
     * @param userPointLog  积分日志
     * @param userGrowthLog 成长值日志
     */
    private void saveLogToKafka(UserPointLog userPointLog, UserGrowthLog userGrowthLog) {

        kafkaSender.send("userPointLog", JSON.toJSONString(userPointLog));
        kafkaSender.send("userGrowthLog", JSON.toJSONString(userGrowthLog));

    }


}
