package com.lj.pointgrowth.utils;

import java.lang.reflect.Field;
import java.sql.*;
import java.util.*;

/**
 * @Author: 黄树文
 * @Date: 2019/12/7 13:30
 */
public class SqlUtils {

    public static ArrayList<Map<String, Object>> getResultAsHashMap(ResultSet resultSet) throws SQLException {
        ResultSetMetaData metaData = resultSet.getMetaData();
        System.out.println(metaData.getColumnCount());
        ArrayList<Map<String, Object>> resultMapArrayList = new ArrayList<>();
        while (resultSet.next()) {
            HashMap<String, Object> resultHashMap = new HashMap<>();
            for (int index = 1; index <= metaData.getColumnCount(); index++) {
                resultHashMap.put(metaData.getColumnName(index), resultSet.getString(metaData.getColumnName(index)));
            }
            resultMapArrayList.add(resultHashMap);
        }
        return resultMapArrayList;
    }

    public static <T> List<T> queryList(Class<T> t, ResultSet rs) {
        List<T> list = new ArrayList<>();
        T obj = null;
        try {

            // 获取插叙结果集中的元数据(获取列类型，数量以及长度等信息)
            ResultSetMetaData rsmd = rs.getMetaData();
            // 声明一个map集合，用于临时存储查询到的一条数据（key：列名；value：列值）
            Map<String, Object> map = new HashMap<>();
            // 遍历结果集
            while (rs.next()) {
                // 防止缓存上一条数据
                map.clear();
                // 遍历所有的列
                for (int i = 0; i < rsmd.getColumnCount(); i++) {
                    // 获取列名
                    String cname = rsmd.getColumnLabel(i + 1);
                    //获取列类型的int表示形式，以及列类型名称
//					System.out.println("列类型:"+rsmd.getColumnType(i + 1)+"----"+rsmd.getColumnTypeName(i+1));
                    // 获取列值
                    Object value = rs.getObject(cname);
                    // 将列明与列值存储到map中
                    map.put(cname, value);
                }
                // 利用反射将map中的数据注入到Java对象中，并将对象存入集合
                if (!map.isEmpty()) {
                    // 获取map集合键集(列名集合)
                    Set<String> columnNames = map.keySet();
                    // 创建对象
                    obj = t.newInstance();//new Student() //java.lang.Object
                    for (String column : columnNames) {
                        // 根据键获取值
                        Object value = map.get(column);
                        //当数据对象不为空时，才注入数据到属性中
                        if (Objects.nonNull(value)) {
                            // 获取属性对象
                            Field f = t.getDeclaredField(column);
                            // 设置属性为可访问状态
                            f.setAccessible(true);
                            // 为属性设置
                            f.set(obj, value);
                        }
                    }
                    list.add(obj);
                }
            }
        } catch (SQLException | SecurityException | InstantiationException | IllegalAccessException | NoSuchFieldException e) {
            e.printStackTrace();
        }
        return list;
    }

    public static <T> List<T> querList(Class<T> t, Connection conn, String sql, Object... params) throws SQLException, IllegalAccessException, InstantiationException, NoSuchFieldException {
        List<T> list = new ArrayList<>();
        T obj = null;

        PreparedStatement ps = conn.prepareStatement(sql);
        for (int i = 0; i < params.length; i++) {
            ps.setObject(i + 1, params[i]);
        }
        ResultSet resultSet = ps.executeQuery();
        ResultSetMetaData metaData = resultSet.getMetaData();
        int columnCount = metaData.getColumnCount();

        ArrayList<T> ts = new ArrayList<>();
        HashMap<String, Object> map = new HashMap<>();
        while (resultSet.next()) {
            map.clear();
            for (int i = 1; i <= columnCount; i++) {
                String columnLabel = metaData.getColumnLabel(i);
                Object value = resultSet.getObject(columnLabel);
                map.put(columnLabel, value);
            }
            if (map != null) {
                Set<String> columnNames = map.keySet();
                obj = t.newInstance();
                for (String columnName : columnNames) {
                    Object value = map.get(columnName);
                    if (value != null) {
                        Field f = t.getDeclaredField(columnName);
                        f.setAccessible(true);
                        f.set(obj, value);
                    }
                }
            }
            list.add(obj);

        }
        resultSet.close();
        ps.close();
        conn.close();
        return list;
    }

    public static <T> List<T> querOneFieldList(Class<T> t, Connection conn, String sql) throws SQLException, IllegalAccessException, InstantiationException, NoSuchFieldException {
        List<T> list = new ArrayList<>();

        PreparedStatement ps = conn.prepareStatement(sql);
//        ps.setObject(1, field);
        ResultSet resultSet = ps.executeQuery();

        while (resultSet.next()) {
            T obj = (T) resultSet.getObject(1);
            list.add(obj);
        }
        return list;
    }
}
